        # print(centre.get())

        # profile = Profile().from_pulse(centre.get(), width.get(),
        #                                (start.get(), end.get()), increment.get())
        # self.subplot.plot(profile.x, profile.y)
        # self.canvas.draw()
